// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//


#include "fi_console"
#include "fi_events"
#include "fi_message"

#include "nwnx_area"
#include "nwnx_object"


// -----------------------------------------------------------------------------

// Return TRUE if oObject is an Area
int GetIsArea(object oArea);
int GetIsArea(object oArea)
{
  if (!GetIsObjectValid(oArea))
    return FALSE;

  return (NWNX_Object_GetInternalObjectType(oArea) == NWNX_OBJECT_TYPE_INTERNAL_AREA);
}


// -----------------------------------------------------------------------------

// Returns TRUE if at least one PC is mid-transition
int GetHasPlayerInTransition();
int GetHasPlayerInTransition()
{
  object oPC = GetFirstPC();
  while (oPC != OBJECT_INVALID)
  {
    object oArea = GetArea(oPC);
    if (!GetIsObjectValid(oArea))
      return TRUE;
    oPC = GetNextPC();
  }
  return FALSE;
}


// -----------------------------------------------------------------------------

// Returns TRUE if at least one PC is mid-transition
int GetHasPlayerInArea(object oArea);
int GetHasPlayerInArea(object oArea)
{
  object oPC = GetFirstPC();
  while (oPC != OBJECT_INVALID)
  {
    if (GetArea(oPC) == oArea)
      return TRUE;
    oPC = GetNextPC();
  }
  return FALSE;
}


// -----------------------------------------------------------------------------
void _WarnAndNotify(string sLog, object oNotify = OBJECT_INVALID);
void _WarnAndNotify(string sLog, object oNotify = OBJECT_INVALID)
{
  LogWarning(sLog);
  SendServerMessage(oNotify, "error: " + sLog, MESSAGE_COLOR_FAILURE);
}


// -----------------------------------------------------------------------------
void _ErrorAndNotify(string sLog, object oNotify = OBJECT_INVALID);
void _ErrorAndNotify(string sLog, object oNotify = OBJECT_INVALID)
{
  LogError(sLog);
  SendServerMessage(oNotify, "error: " + sLog, MESSAGE_COLOR_FAILURE);
}


// -----------------------------------------------------------------------------
int _DestroyAreaEx(string sResRef, object oNotify = OBJECT_INVALID);
int _DestroyAreaEx(string sResRef, object oNotify = OBJECT_INVALID)
{
  if (sResRef == "")
    return FALSE;

  object oArea = GetObjectByTag(sResRef);
  if (!GetIsObjectValid(oArea))
    return FALSE;

  if (!GetIsArea(oArea))
    return FALSE;

  string sName = GetName(oArea);
  string sInfo = "'" + GetName(oArea) + "' (" + sResRef + ")";
  object oStarting = GetAreaFromLocation(GetStartingLocation());

  if (oArea == oStarting)             { _WarnAndNotify("Refusing to recreate area with starting location", oNotify); return FALSE; }
  if (GetHasPlayerInTransition())     { _WarnAndNotify("Refusing to destroy area, players are in transition.", oNotify); return FALSE; }
  if (GetHasPlayerInArea(oArea))      { _WarnAndNotify("Refusing to recreate area " + sInfo + ": players are in the area", oNotify) ; return FALSE; }

  BroadcastEvent(EVENT_AREA_DESTROY_BEFORE, oArea);
  int nVal = DestroyArea(oArea);
  if (nVal == -2) { _WarnAndNotify("Failed to destroy area " + sInfo + ": players are in area", oNotify);  return FALSE; }
  if (nVal < 0)   { _ErrorAndNotify("Failed to destroy area " + sInfo, oNotify); return FALSE; }
  LogInfo("Destroyed area '" + sName + "' (" + sResRef + ").", CONSOLE_COLOR_GREEN);

  return TRUE;
}


// -----------------------------------------------------------------------------
object _CreateAreaEx(string sResRef, object oNotify = OBJECT_INVALID);
object _CreateAreaEx(string sResRef, object oNotify = OBJECT_INVALID)
{
  if (sResRef == "")
    return OBJECT_INVALID;

  object oArea = CreateArea(sResRef, sResRef);
  if (oArea == OBJECT_INVALID)
  {
    _ErrorAndNotify("Failed to create area '" + sResRef + "'.", oNotify);
    return OBJECT_INVALID;
  }

  LogInfo("Created area '" + GetName(oArea) + "' (" + sResRef + ").", CONSOLE_COLOR_GREEN);
  BroadcastEvent(EVENT_AREA_CREATED, oArea);
  return oArea;
}


// -----------------------------------------------------------------------------
object RecreateArea(string sResRef, object oNotify = OBJECT_INVALID);
object RecreateArea(string sResRef, object oNotify = OBJECT_INVALID)
{
  if (sResRef == "")
    return OBJECT_INVALID;

  object oArea = GetObjectByTag(sResRef);
  if (GetIsObjectValid(oArea))
  {
    if (!_DestroyAreaEx(sResRef, oNotify))
      return OBJECT_INVALID;
  }
  else
    SendServerMessage(oNotify, "warning: area '" + sResRef + "' does not exist.", MESSAGE_COLOR_FAILURE);

  oArea = _CreateAreaEx(sResRef, oNotify);
  if (GetIsObjectValid(oArea))
    SendServerMessage(oNotify, "success: Recreated area '" + GetName(oArea) + "' (" + sResRef + ").", MESSAGE_COLOR_SUCCESS);

  return oArea;
}

// void main() {}
