// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Handle logging to the console log

#include "fi_debug"

#include "nwnx_admin"
#include "nwnx_util"


// Linux console ANSI color codes
const string CONSOLE_COLOR_NONE   = "[00m";
const string CONSOLE_COLOR_RED    = "[01;31m";
const string CONSOLE_COLOR_GREEN  = "[01;32m";
const string CONSOLE_COLOR_YELLOW = "[01;33m";
const string CONSOLE_COLOR_BLUE   = "[01;34m";
const string CONSOLE_COLOR_PURPLE = "[01;35m";
const string CONSOLE_COLOR_CYAN   = "[01;36m";
const string CONSOLE_COLOR_WHITE  = "[01;37m";


//! @brief Return true if console is a TTY
int IsATTY();
int IsATTY()
{
  return NWNX_Util_GetTTY() == "" ? FALSE : TRUE;
}


//! @brief Return the famous "\e" aka "'\033"
string GetConsoleASCIIEscape();
string GetConsoleASCIIEscape()
{

  return GetStringRight(GetStringLeft(NWNX_Util_GetAsciiTableString(), 28), 1);
}


//! @brief Return a Linux shell color control code CONSOLE_COLOR_* if TTY, otherwise return the uncolored string.
//! @param sString       String to colorize
//! @param sConsoleColor A CONSOLE_COLOR_*
//! @returns Colorized string
string GetConsoleColorString(string sString, string sConsoleColor);
string GetConsoleColorString(string sString, string sConsoleColor)
{
  if (!IsATTY())
    return sString;

  return   GetConsoleASCIIEscape() + sConsoleColor
         + sString
         + GetConsoleASCIIEscape() + CONSOLE_COLOR_NONE;
}


//! @brief Clear the screen
void ConsoleClearScreen();
void ConsoleClearScreen()
{
  PrintString(GetConsoleASCIIEscape() + "c");
}


//! @brief Write sLogEntry as a timestamped entry into the log file.
//! @param sPrefix       Add this prefix to the string
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void WriteTimestampedLogEntryEx(string sPrefix, string sLogEntry, string sConsoleColor = CONSOLE_COLOR_NONE);
void WriteTimestampedLogEntryEx(string sPrefix, string sLogEntry, string sConsoleColor = CONSOLE_COLOR_NONE)
{
  string sString = "";
  if (GetStringLength(sPrefix))
    sString = sPrefix + " ";

  sString += sLogEntry;

  if (sConsoleColor != CONSOLE_COLOR_NONE)
    sString = GetConsoleColorString(sString, sConsoleColor);

  WriteTimestampedLogEntry(sString);
}


//! @brief Write sLogEntry as a timestamped entry into the log file.
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void LogDebug(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_BLUE);
void LogDebug(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_BLUE)
{
  if (GetIsDebug())
    WriteTimestampedLogEntryEx("debug:", sLogEntry, sConsoleColor);
}


//! @brief Write sLogEntry as a prefixed timestamped entry into the log file.
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void LogInfo(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_NONE);
void LogInfo(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_NONE)
{
  WriteTimestampedLogEntryEx("info:", sLogEntry, sConsoleColor);
}


//! @brief Write sLogEntry as a prefixed timestamped entry into the log file, in color.
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void LogNotice(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_GREEN);
void LogNotice(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_GREEN)
{
  WriteTimestampedLogEntryEx("notice:", sLogEntry, sConsoleColor);
}


//! @brief Write sLogEntry as a prefixed timestamped entry into the log file, in color.
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void LogWarning(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_YELLOW);
void LogWarning(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_YELLOW)
{
  WriteTimestampedLogEntryEx("warning:", sLogEntry, CONSOLE_COLOR_YELLOW);
}


//! @brief Write sLogEntry as a prefixed timestamped entry into the log file, in color.
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void LogError(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_RED);
void LogError(string sLogEntry, string sConsoleColor = CONSOLE_COLOR_RED)
{
  WriteTimestampedLogEntryEx("error:", sLogEntry, CONSOLE_COLOR_RED);
}


//! @brief Write sLogEntry as a prefixed timestamped entry into the log file, in color; then shutdown.
//! @param sLogEntry     Print this log entry
//! @param sConsoleColor A CONSOLE_COLOR_*
void LogFatal(string sLogEntry);
void LogFatal(string sLogEntry)
{
  LogError(sLogEntry);
  LogStackTrace();
  SetLocalInt(GetModule(), NWN_SHUTDOWN, 1);
  NWNX_Administration_ShutdownServer();
}
