// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//


#include "fi_const"
#include "fi_envvar"

#include "nwnx_object"
#include "nwnx_encounter"


// (bool)
// Constant used on module, areas, and/or PC
const string DEBUG = "DEBUG";


// -----------------------------------------------------------------------------

// Return TRUE value if any of oObject, area of oObject, or module has DEBUG set.
// Return TRUE if envvar NWN_DEBUG is set
int GetIsDebug(object oObject = OBJECT_INVALID);
int GetIsDebug(object oObject = OBJECT_INVALID)
{
  if (GetLocalInt(oObject, DEBUG))
    return TRUE;
  if (GetLocalInt(GetArea(oObject), DEBUG))
    return TRUE;
  if (GetLocalInt(GetModule(), DEBUG))
    return TRUE;
  if (GetEnvironmentVariableAsBool(NWN_DEBUG))
    return TRUE;
  return FALSE;
}


// -----------------------------------------------------------------------------

// Return a string representing the object type.
string GetObjectTypeString(object oObject);
string GetObjectTypeString(object oObject)
{
  if (GetIsDM(oObject))
    return "DM";
  if (GetIsPC(oObject))
    return "PC";

  if (!GetIsObjectValid(oObject))
   return "invalid";

  switch (NWNX_Object_GetInternalObjectType(oObject))
  {
    case NWNX_OBJECT_TYPE_INTERNAL_INVALID:      return "invalid";
    case NWNX_OBJECT_TYPE_INTERNAL_AREA:         return "area";
    case NWNX_OBJECT_TYPE_INTERNAL_AREAOFEFFECT: return "AOE";
    case NWNX_OBJECT_TYPE_INTERNAL_CREATURE:     return "creature";
    case NWNX_OBJECT_TYPE_INTERNAL_DOOR:         return "door";
    case NWNX_OBJECT_TYPE_INTERNAL_ENCOUNTER:    return "encounter";
    case NWNX_OBJECT_TYPE_INTERNAL_ITEM:         return "item";
    case NWNX_OBJECT_TYPE_INTERNAL_MODULE:       return "module";
    case NWNX_OBJECT_TYPE_INTERNAL_PLACEABLE:    return "placeable";
    case NWNX_OBJECT_TYPE_INTERNAL_SOUND:        return "sound";
    case NWNX_OBJECT_TYPE_INTERNAL_STORE:        return "store";
    case NWNX_OBJECT_TYPE_INTERNAL_TRIGGER:      return "trigger";
    case NWNX_OBJECT_TYPE_INTERNAL_WAYPOINT:     return "waypoint";
  }

  return "<error>";
}


// -----------------------------------------------------------------------------

// Return information on oObject unless it's invalid.
string GetObjectInfo(object oObject);
string GetObjectInfo(object oObject)
{
  if (oObject == OBJECT_INVALID)
    return "<OBJECT_INVALID>";

  if (!GetIsObjectValid(oObject))
    return "<invalid>";

  int nType        = NWNX_Object_GetInternalObjectType(oObject);
  string sName     = GetName(oObject);
  string sTrueName = GetName(oObject, TRUE);
  string sTag      = GetTag(oObject);
  string sRefRef   = GetResRef(oObject);

  if (NWNX_OBJECT_TYPE_INTERNAL_SOUND == nType)
    return "<sound> (tag='" + sTag + "')";

  string sMessage =
    ((sName == "") ? "<empty>" : "'" + sName + "'")
    + " (" + GetObjectTypeString(oObject)
    + (sTag != "" ? ", tag='" + sTag + "'" : "");

  if (sTrueName != sName && sTrueName != "")
    sMessage += ", initial-name='" + sTrueName + "'";

  if (GetIsPC(oObject))
  {
    sMessage += ", level=" + IntToString(GetHitDice(oObject));
    sMessage += ", GetPCPublicCDKey=" + GetPCPublicCDKey(oObject);
  }

  if (sRefRef != "")
    sMessage += ", resref='" + sRefRef + "'";

  // Get details if possible
  switch (GetObjectType(oObject))
  {
    case OBJECT_TYPE_ENCOUNTER:
      sMessage += ", nMin: " + IntToString(NWNX_Encounter_GetMinNumSpawned(oObject));
      sMessage += ", nMax: " + IntToString(NWNX_Encounter_GetMaxNumSpawned(oObject));
      break;
  }

  // Get the container to add details if possible
  switch (GetObjectType(GetItemPossessor(oObject)))
  {
    case OBJECT_TYPE_STORE:
      sMessage += ", infinite=" + (GetInfiniteFlag(oObject) ? "1" : "0");
      break;
  }

  sMessage += ")";
  return sMessage ;
}


// -----------------------------------------------------------------------------

// Write all string and int variables to console.
void LogLocalVars(object oObject);
void LogLocalVars(object oObject)
{
  int nCount = NWNX_Object_GetLocalVariableCount(oObject);

  int i;
  for (i = 0; i <= nCount; i++)
  {
    struct NWNX_Object_LocalVariable stVar = NWNX_Object_GetLocalVariable(oObject, i);

    switch (stVar.type)
    {
      case NWNX_OBJECT_LOCALVAR_TYPE_INT:
        WriteTimestampedLogEntry("[var] " + stVar.key + ": " + IntToString(GetLocalInt(oObject, stVar.key)));
        break;
      case NWNX_OBJECT_LOCALVAR_TYPE_FLOAT:
        WriteTimestampedLogEntry("[var] " + stVar.key + ": " + FloatToString(GetLocalFloat(oObject, stVar.key), 7, 5));
        break;
      case NWNX_OBJECT_LOCALVAR_TYPE_STRING:
        WriteTimestampedLogEntry("[var] " + stVar.key + ": " + GetLocalString(oObject, stVar.key));
        break;
      case NWNX_OBJECT_LOCALVAR_TYPE_OBJECT:
        WriteTimestampedLogEntry("[var] " + stVar.key + ": " + GetObjectInfo(GetLocalObject(oObject, stVar.key)));
        break;
      case NWNX_OBJECT_LOCALVAR_TYPE_LOCATION:
        WriteTimestampedLogEntry("[var] " + stVar.key + ": <LOCATION>");
        break;
      case NWNX_OBJECT_LOCALVAR_TYPE_JSON:
        WriteTimestampedLogEntry("[var] " + stVar.key + ": <JSON>");
        break;
      break;
    }

  }
}


// -----------------------------------------------------------------------------

// Write sLogEntry as a prefixed timestamped entry into the log file, in color.
void LogStackTrace();
void LogStackTrace()
{
  int nDepth = 0;
  string sTrace = NWNX_Util_GetCurrentScriptName(nDepth);
  while (sTrace != "")
  {
    string sMessage = "[trace] " + IntToString(nDepth) + ": " + sTrace;
    WriteTimestampedLogEntry(sMessage);
    sTrace = NWNX_Util_GetCurrentScriptName(++nDepth);
  }
}

// void main() {}
