// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Handle encounters and creature generation.


#include "nwnx_encounter"


//! @brief Get the minimun CR from encounter
//! @param oEncounter An encounter
//! @returns An integer as a float
float GetEncounterMinCR(object oEncounter);
float GetEncounterMinCR(object oEncounter)
{
  int nSet = 0;
  float fCR = 0.0f;
  int nIndex = 0;
  struct NWNX_Encounter_CreatureListEntry sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, nIndex);
  while (sListEntry.resref != "")
  {
    if (nSet == 0)
    {
      fCR = sListEntry.challengeRating;
      nSet = 1;
    } else {
      if (sListEntry.challengeRating < fCR)
        fCR = sListEntry.challengeRating;
    }
    sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, ++nIndex);
  }
  return fCR;
}


//! @brief Get the maximum CR from encounter
//! @param oEncounter An encounter
//! @returns An integer as a float
float GetEncounterMaxCR(object oEncounter);
float GetEncounterMaxCR(object oEncounter)
{
  float fCR = 0.0f;
  int nIndex = 0;
  struct NWNX_Encounter_CreatureListEntry sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, nIndex);
  while (sListEntry.resref != "")
  {
    if (sListEntry.challengeRating > fCR)
      fCR = sListEntry.challengeRating;
    sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, ++nIndex);
  }
  return fCR;
}


//! @brief Reset the alreadyUsed flag of all creatures in encounter
//! @param oEncounter An encounter
void ResetEncounterAlreadyUsed(object oEncounter);
void ResetEncounterAlreadyUsed(object oEncounter)
{
  // Reset the encounter
  int nIndex = 0;
  struct NWNX_Encounter_CreatureListEntry sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, nIndex);
  while (sListEntry.resref != "")
  {
    sListEntry.alreadyUsed = FALSE;
    NWNX_Encounter_SetEncounterCreatureByIndex(oEncounter, nIndex, sListEntry);
    sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, ++nIndex);
  }
}


//! @brief Translate the encounter as a json structure.
//! @brief
//! @brief Possible fields: index (int), resref (string), challengeRating (float)
//! @brief unique (int), alreadyUsed (int)
//! @param oEncounter An encounter
//! @returns A json array of encounter structure
json GetEncounterAsJson(object oEncounter);
json GetEncounterAsJson(object oEncounter)
{
  // Convert encounter to json
  json jEncounter = JsonArray();

  int nIndex = 0;
  struct NWNX_Encounter_CreatureListEntry sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, nIndex);
  while (sListEntry.resref != "")
  {
    json jListEntry = JsonObject();
    jListEntry = JsonObjectSet(jListEntry, "index", JsonInt(nIndex));
    jListEntry = JsonObjectSet(jListEntry, "resref", JsonString(sListEntry.resref));
    jListEntry = JsonObjectSet(jListEntry, "challengeRating", JsonFloat(sListEntry.challengeRating));
    jListEntry = JsonObjectSet(jListEntry, "unique", JsonInt(sListEntry.unique));
    jListEntry = JsonObjectSet(jListEntry, "alreadyUsed", JsonInt(sListEntry.alreadyUsed));

    jEncounter = JsonArrayInsert(jEncounter, jListEntry);
    sListEntry = NWNX_Encounter_GetEncounterCreatureByIndex(oEncounter, ++nIndex);
  }
  return jEncounter;
}


//! @brief Return index of a pseudo-random creature from json array.
//! @param jEncounter An json encounter
//! @param fCR Approximative CR of creature to return.
//! @returns Index of creature to use
int GetEncounterCreature(json jEncounter, float fCR);
int GetEncounterCreature(json jEncounter, float fCR)
{
  int nLength = JsonGetLength(jEncounter);
  if (nLength == 0)
    return -1;

  // Cut the jEncounter to all those closest to fCR.
  int nIndex = 0;
  int nFound = FALSE;
  float fCRMaxDiff = 0.0f;
  json jCandidates = JsonArray();

  // 300 instructions per loop
  while (nIndex < nLength)
  {
    json jEntry = JsonArrayGet(jEncounter, nIndex);
    float fCRDiff = fabs(fCR - JsonGetFloat(JsonObjectGet(jEntry, "challengeRating")));

    if (nFound == FALSE)
    {
      nFound = TRUE;
      fCRMaxDiff = fCRDiff;
      jCandidates = JsonArrayInsert(jCandidates, jEntry);
      // LogInfo(" -> Initial creature: " + JsonGetString(JsonObjectGet(jEntry, "resref")));
    }
    else if (fCRDiff < fCRMaxDiff)
    {
      // Replace
      fCRMaxDiff = fCRDiff;
      jCandidates = JsonArray();
      jCandidates = JsonArrayInsert(jCandidates, jEntry);
      // LogInfo(" -> Replacement creature: " + JsonGetString(JsonObjectGet(jEntry, "resref")));
    }
    else if (fCRDiff == fCRMaxDiff)
    {
      // Replace
      jCandidates = JsonArrayInsert(jCandidates, jEntry);
    }
    nIndex += 1;
  }

  int nRandom = Random(JsonGetLength(jCandidates));
  json jEntry = JsonArrayGet(jCandidates, nRandom);
  return JsonGetInt(JsonObjectGet(jEntry, "index"));
}


//! @brief Create a single creature from encounter.
//! @param jEncounter An json encounter
//! @param fCR Approximative CR of creature to return.
//! @returns Creature created
object CreateCreatureFromEncounter(object oEncounter, float fCR, location lWhere);
object CreateCreatureFromEncounter(object oEncounter, float fCR, location lWhere)
{
  ResetEncounterAlreadyUsed(oEncounter);

  json jEncounter = GetEncounterAsJson(oEncounter);
  int nIndex = GetEncounterCreature(jEncounter, fCR);

  json jEntry = JsonArrayGet(jEncounter, nIndex);
  string sResRef = JsonGetString(JsonObjectGet(jEntry, "resref"));

  object oCreature = CreateObject(OBJECT_TYPE_CREATURE, sResRef, lWhere, FALSE, sResRef);
  return oCreature;
}
// void main() {}
