// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//
// Broadcast and execute scripts inline
// Partially replaces x2_inc_switches and signals


#include "fi_const"
#include "fi_console"
#include "fi_events_name"

#include "inc_array"
#include "nwnx_events"
#include "nwnx_util"


const int X2_EXECUTE_SCRIPT_FAILURE = -1;
const int X2_EXECUTE_SCRIPT_MISSING = -2;

const string X2_L_LAST_ITEM_EVENT = "X2_L_LAST_ITEM_EVENT";
const string X2_L_LAST_USER_DEFINED_EVENT = "X2_L_LAST_USER_DEFINED_EVENT";

const string _REGISTER_EVENT_ID = "_REGISTER_EVENT_ID";
const string _REGISTER_EVENT_SCRIPT = "_REGISTER_EVENT_SCRIPT";


// -----------------------------------------------------------------------------

// Register a single event for a unit
void RegisterEvent(int nEventId, string sUnit = "");
void RegisterEvent(int nEventId, string sUnit = "")
{
  object oModule = GetModule();

  if (sUnit == "")
    sUnit = NWNX_Util_GetCurrentScriptName();

  SetLocalInt(oModule, _REGISTER_EVENT_ID, nEventId);
  SetLocalString(oModule, _REGISTER_EVENT_SCRIPT, sUnit);
  // Do not call ExpectScriptToExist since we're iterating on ResRef.
  ExecuteScript(SCRIPT_FW_EVT_SUBSCRIBE, OBJECT_SELF);
  DeleteLocalInt(oModule, _REGISTER_EVENT_ID);
  DeleteLocalString(oModule, _REGISTER_EVENT_SCRIPT);
}


// -----------------------------------------------------------------------------

// Execute a named event script with a nEvent integer passed as a variable
// This is processed inline (ie: it uses ExecuteScript and not SignalEvent)
//
// Retrieve nEvent integer with GetUserDefinedItemEventNumber
// Returns one of X2_EXECUTE_SCRIPT_* with X2_EXECUTE_SCRIPT_FAILURE usually
// meaning to continue
// cf. SetExecutedScriptReturnValue
int ExecuteEventScript(string sScript, int nEventId, object oTarget = OBJECT_SELF);
int ExecuteEventScript(string sScript, int nEventId, object oTarget = OBJECT_SELF)
{
  if (sScript == "")
    return X2_EXECUTE_SCRIPT_MISSING;

  SetLocalInt(oTarget, X2_L_LAST_USER_DEFINED_EVENT, nEventId);
  return ExecuteScriptAndReturnInt(sScript, oTarget);
}


// -----------------------------------------------------------------------------

// Send a signal to all registered units.
// oObjectSelf is the object that becomes OBJECT_SELF
void BroadcastEvent(int nEventId, object oSelf = OBJECT_SELF);
void BroadcastEvent(int nEventId, object oSelf = OBJECT_SELF)
{
  // Some events have an invalid OBJECT_SELF
  if (!GetIsObjectValid(oSelf))
    oSelf = GetModule();

  if (!nEventId)
  {
    LogError("Recieved an event with empty nEventId (fi_events)");
    return;
  }

  string sEventName = GetEventName(nEventId);
  if (sEventName == "")
  {
    LogError("Recieved an event with empty sEventName for nEvent " + IntToString(nEventId) + " (fi_events)");
    return;
  }

  int nIndex = 0;
  string sUnit = Array_At_Str(sEventName, nIndex);
  while (sUnit != "")
  {
    ExecuteEventScript(sUnit, nEventId, oSelf);
    sUnit = Array_At_Str(sEventName, ++nIndex);
  }
}


// -----------------------------------------------------------------------------

// Get the current UserDefined Call Event Number, or if missing, the
// GetUserDefinedEventNumber as fallback
// Not to be confused with GetUserDefinedItemEventNumber (for items only)
// cf. GetUserDefinedEventParam, ExecuteEventScript
int GetEvent(object oTarget = OBJECT_SELF);
int GetEvent(object oTarget = OBJECT_SELF)
{
  // Delete of the variable is done by ExecuteEventScript
  int nRet = GetLocalInt(oTarget, X2_L_LAST_USER_DEFINED_EVENT);
  if (nRet)
    return nRet;
  return GetUserDefinedEventNumber();
}


// -----------------------------------------------------------------------------

void RegisterServerConsoleCommand(string sEvent);
void RegisterServerConsoleCommand(string sEvent)
{
  string SCRIPT_CHUNK_CONSOLE = "SetLocalString(GetModule(), \"ARGS\", \"$args\"); ExecuteScript(\"fw_evt_console\");";
  SCRIPT_CHUNK_CONSOLE = "SetLocalString(GetModule(), \"COMMAND\", \"" + sEvent + "\"); " + SCRIPT_CHUNK_CONSOLE;

  NWNX_Util_RegisterServerConsoleCommand(sEvent, SCRIPT_CHUNK_CONSOLE);
}


// -----------------------------------------------------------------------------

string GetConsoleCommand();
string GetConsoleCommand()
{
  return GetLocalString(GetModule(), "COMMAND");
}

// -----------------------------------------------------------------------------

string GetConsoleCommandArgs();
string GetConsoleCommandArgs()
{
  return GetLocalString(GetModule(), "ARGS");
}

// void main() {}
