// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Handle inventory

#include "fi_object"


//! @brief Return object with given tag
//! @param oTarget Target inventory
//! @param sTag Tag of item to search for
//! @returns The item
object GetItemFromInventory(object oTarget, string sTag);
object GetItemFromInventory(object oTarget, string sTag)
{
  object oItem = GetFirstItemInInventory(oTarget);
  while (oItem != OBJECT_INVALID)
  {
    if (GetTag(oItem) == sTag)
      return oItem;
    oItem =  GetNextItemInInventory(oTarget);
  }
  return OBJECT_INVALID;
}


//! @brief Destroy oTarget inventory (irrevocably).
//! @param oTarget Target inventory
//! @param fDelay Delay before deletion
void DestroyInventory(object oTarget, float fDelay = 0.0f);
void DestroyInventory(object oTarget, float fDelay = 0.0f)
{
  object oItem = GetFirstItemInInventory(oTarget);
  while (oItem != OBJECT_INVALID)
  {
    DestroyObjectEx(oItem, fDelay);
    oItem = GetNextItemInInventory(oTarget);
  }
}


//! @brief Return TRUE if object with given tag exists in inventory
//! @param oTarget Target inventory
//! @param sTag Tag of item to search for
//! @returns Boolean
int HasItemInInventory(object oTarget, string sTag)
{
  object oItem = GetItemFromInventory(oTarget, sTag);
  return GetIsObjectValid(oItem);
}


//! @brief Returns count of objects with given tag
//! @param oTarget Target inventory
//! @param sTag Tag of item to search for
//! @returns Total count
int CountItemsInInventory(object oTarget, string sTag);
int CountItemsInInventory(object oTarget, string sTag)
{
  int nCount = 0;
  object oItem = GetFirstItemInInventory(oTarget);
  while (oItem != OBJECT_INVALID)
  {
    if (GetTag(oItem) == sTag)
      nCount += 1;
    oItem =  GetNextItemInInventory(oTarget);
  }
  return nCount;
}


//! @brief Returns count of objects with given tag
//! @param oTarget Target inventory
//! @param nCount Maximum number of items to destroy
//! @param sTag Tag of item to search for
void DestroyItemInInventory(object oTarget, string sTag, int nCount = 1);
void DestroyItemInInventory(object oTarget, string sTag, int nCount = 1)
{
  object oItem = GetFirstItemInInventory(oTarget);
  while (oItem != OBJECT_INVALID)
  {
    if (GetTag(oItem) == sTag)
    {
      DestroyObject(oItem);
      nCount -= 1;
      if (nCount == 0)
        return;
    }
    oItem =  GetNextItemInInventory(oTarget);
  }
}


// void main() {}
