// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//


#include "nwnx_object"


// -----------------------------------------------------------------------------

const int PLACEABLE_STATE_INVALID     = -1;
const int PLACEABLE_STATE_OPEN        =  4055;
const int PLACEABLE_STATE_CLOSE       = -4055;
const int PLACEABLE_STATE_ACTIVATED   = ANIMATION_PLACEABLE_ACTIVATE;
const int PLACEABLE_STATE_DEACTIVATED = ANIMATION_PLACEABLE_DEACTIVATE;

const string X2_L_PLC_ACTIVATED_STATE = "X2_L_PLC_ACTIVATED_STATE";
const string X2_L_PLC_OPEN_STATE      = "X2_L_PLC_OPEN_STATE";


// -----------------------------------------------------------------------------

// Get the first object in oArea that has sTag
object GetObjectInAreaByTag(string sTag, object oArea);
object GetObjectInAreaByTag(string sTag, object oArea)
{
  // Get any object first
  object oFirst = GetFirstObjectInArea(oArea);
  if (!GetIsObjectValid(oFirst))
    return OBJECT_INVALID;
  if (GetTag(oFirst) == sTag)
    return oFirst;

  return GetNearestObjectByTag(sTag, oFirst);
}


// -----------------------------------------------------------------------------

void DestroyInventory(object oTarget, float fDelay = 0.0f);

// Destroy oObject (irrevocably).
// This will not work on modules and areas.
object DestroyObjectEx(object oObject, float fDelay = 0.0f);
object DestroyObjectEx(object oObject, float fDelay = 0.0f)
{
  if (GetHasInventory(oObject))
    DestroyInventory(oObject, fDelay);

  SetPlotFlag(oObject, FALSE);
  DestroyObject(oObject, fDelay);
  return OBJECT_INVALID;
}


// -----------------------------------------------------------------------------

// Return TRUE if current state == nEvent;
// Return -1 if error
int GetIsObjectState(object oObject, int nEvent);
int GetIsObjectState(object oObject, int nEvent)
{
  if (!GetIsObjectValid(oObject))
    return -1;

  // https://github.com/nwnxee/unified/blob/master/NWNXLib/API/Constants/Animation.hpp
  //
  //  PlaceableActivated   = 73,
  //  PlaceableDeactivated = 74,
  //  PlaceableOpened      = 75,
  //  PlaceableClosed      = 76,
  switch (nEvent)
  {
    case PLACEABLE_STATE_OPEN:        return (NWNX_Object_GetCurrentAnimation(oObject) == 75);
    case PLACEABLE_STATE_CLOSE:       return (NWNX_Object_GetCurrentAnimation(oObject) == 76);
    case PLACEABLE_STATE_ACTIVATED:   return (NWNX_Object_GetCurrentAnimation(oObject) == 73);
    case PLACEABLE_STATE_DEACTIVATED: return (NWNX_Object_GetCurrentAnimation(oObject) == 74);
  }
  return PLACEABLE_STATE_INVALID;
}


// -----------------------------------------------------------------------------

// Set object state to nEvent STATE_* after interaction with it
// Sets LAST_OPENED_EVENT, LAST_CLOSED_EVENT, LAST_ACTIVATION_EVENT, LAST_DEACTIVATION_EVENT
// Plays animation if needed for open/close, always for activation
// If bQueryState is true, it will check current state before playing animation
void SetObjectState(object oObject, int nEvent);
void SetObjectState(object oObject, int nEvent)
{
  if (!GetIsObjectValid(oObject))
    return;

  switch (nEvent)
  {
    case PLACEABLE_STATE_OPEN:
      if (!GetIsObjectState(oObject, ANIMATION_PLACEABLE_OPEN))
        AssignCommand(oObject, PlayAnimation(ANIMATION_PLACEABLE_OPEN));
      SetLocalInt(oObject, X2_L_PLC_OPEN_STATE, TRUE);
      break;

    case PLACEABLE_STATE_CLOSE:
      if (!GetIsObjectState(oObject, ANIMATION_PLACEABLE_CLOSE))
        AssignCommand(oObject, PlayAnimation(ANIMATION_PLACEABLE_CLOSE));
      SetLocalInt(oObject, X2_L_PLC_OPEN_STATE, FALSE);
      break;

    case PLACEABLE_STATE_ACTIVATED:
      if (!GetIsObjectState(oObject, ANIMATION_PLACEABLE_ACTIVATE))
        AssignCommand(oObject, PlayAnimation(ANIMATION_PLACEABLE_ACTIVATE));
      SetLocalInt(oObject, "NW_L_AMION", TRUE);
      SetLocalInt(oObject, X2_L_PLC_ACTIVATED_STATE, TRUE);
      break;

    case PLACEABLE_STATE_DEACTIVATED:
      if (!GetIsObjectState(oObject, ANIMATION_PLACEABLE_DEACTIVATE))
        AssignCommand(oObject, PlayAnimation(ANIMATION_PLACEABLE_DEACTIVATE));
      SetLocalInt(oObject, "NW_L_AMION", FALSE);
      SetLocalInt(oObject, X2_L_PLC_ACTIVATED_STATE, FALSE);
      break;
  }
}

// Get the nNth object with the specified tag.
// - sTag
// * Returns OBJECT_INVALID if the object cannot be found.
object GetObjectByTagInArea(string sTag, object oArea);
object GetObjectByTagInArea(string sTag, object oArea)
{
  int nIndex = 0;
  object oObject = GetObjectByTag(sTag);
  while (GetIsObjectValid(oObject))
  {
    if (GetArea(oObject) == oArea)
      return oObject;
    oObject = GetObjectByTag(sTag, ++nIndex);
  }
  return OBJECT_INVALID;
}



// void main() {}
