// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! PC functions


#include "fi_const"
#include "fi_class"
#include "fi_console"

#include "nwnx_object"


//! @brief Return TRUE if and only if oObject is only a PC.
//! @param oObject Object that might be a PC
//! @returns TRUE or FALSE
int GetIsPCOnly(object oObject);
int GetIsPCOnly(object oObject)
{
  if (GetIsDM(oObject))
    return FALSE;
  if (GetIsPossessedFamiliar(oObject))
    return FALSE;
  if (GetIsDMPossessed(oObject))
    return FALSE;
  return GetIsPC(oObject);
}


//! @brief Get XP required to be of the parameter level
//! @param nLevel Level from 1 to 40
//! @returns 2da value from exptable.2da
int GetXPForLevel(int nLevel);
int GetXPForLevel(int nLevel)
{
  return StringToInt(Get2DAString("exptable", "XP", nLevel - 1));
}


//! @brief Calculate ECL Effective Character Level
//!
//! @brief https://nwn.fandom.com/wiki/Effective_character_level
//! @param nClassPosition Which class to lookup in classes.2da
//! @param oPC The PC
//! @returns 0 if no class.
int GetECLevelByPosition(int nClassPosition, object oPC);
int GetECLevelByPosition(int nClassPosition, object oPC)
{
  if (!GetIsPC(oPC))
    return 0;

  if (GetClassByPosition(nClassPosition, oPC) == CLASS_TYPE_INVALID)
    return 0;

  int nLevel = GetLevelByPosition(nClassPosition, oPC);
  if (nLevel >= 14)
    return nLevel;

  int nClass = GetClassByPosition(nClassPosition, oPC);
  string sValue = Get2DAString("classes", "EffCRLvl" + GetStringRight("00" + IntToString(nLevel), 2), nClass);
  if (sValue == "")
    return nLevel; // assert failed... TODO

  return StringToInt(sValue);
}


//! @brief Return an Effective Character Level, same as GetHitDice but for ECL
//!
//! @brief https://nwn.fandom.com/wiki/Effective_character_level
//! @brief https://nwnlexicon.com/index.php?title=Class_type
//! @param oPC The PC
//! @returns 0 if error
int GetPlayerECLevel(object oPC);
int GetPlayerECLevel(object oPC)
{
  if (!GetIsPC(oPC))
    return 0;

  // Get values
  int nHD = GetHitDice(oPC);
  if (nHD >= 14)
    return nHD;

  int n1_ECL = GetECLevelByPosition(1, oPC);
  int n2_ECL = GetECLevelByPosition(2, oPC);
  int n3_ECL = GetECLevelByPosition(3, oPC);

  int nTotal = n1_ECL + n2_ECL + n3_ECL;
  if (nTotal == nHD)
    return nHD;

  int n1_LVL = GetLevelByPosition(1, oPC);
  int n2_LVL = GetLevelByPosition(2, oPC);
  int n3_LVL = GetLevelByPosition(3, oPC);

  int n1_CLS = GetClassByPosition(1, oPC);
  int n2_CLS = GetClassByPosition(2, oPC);
  int n3_CLS = GetClassByPosition(3, oPC);

  LogDebug("ECL calculations for " + GetObjectInfo(oPC)
    + " => HitDice = " + IntToString(nHD)
    + ", " + GetClassName(n1_CLS) + " = " + IntToString(n1_ECL) + " instead of " + IntToString(n1_LVL)
    + ((n2_CLS != CLASS_TYPE_INVALID) ? ", " + GetClassName(n2_CLS) + " = " + IntToString(n2_ECL) + " instead of " + IntToString(n2_LVL) : "")
    + ((n3_CLS != CLASS_TYPE_INVALID) ? ", " + GetClassName(n3_CLS) + " = " + IntToString(n3_ECL) + " instead of " + IntToString(n3_LVL) : "")
    + ", Total = " + IntToString(nTotal)
    + (nTotal == nHD ? "" : " instead of " + IntToString(nHD))
    );
  return nTotal;
}


//! @brief Cancel a level up/
//! @param oPC The PC
void CancelLevelUp(object oPC);
void CancelLevelUp(object oPC)
{
  int nPreviousXP = GetXP(oPC);
  int nLevelXP = GetXPForLevel(GetHitDice(oPC)) - 1;
  SetXP(oPC, nLevelXP);
  DelayCommand(0.4f, SetXP(oPC, nPreviousXP));
}


// void main() {}
