// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//
// Execute onUserDefined scripts

#include "fi_events"

#include "nwnx_object"

#include "x2_inc_switches"


// -----------------------------------------------------------------------------

// Tag based script execution for items.
// This is processed inline (ie: it uses ExecuteScript and not SignalEvent)
//
// Uses X2_ITEM_EVENT_* type events.
// Retrieve nEvent integer with GetUserDefinedItemEventNumber
// Returns one of X2_EXECUTE_SCRIPT_* with X2_EXECUTE_SCRIPT_FAILURE usually
// meaning to continue
//
int _ExecuteItemScript(string sFileName, object oItem, int nEvent)
{
  if (sFileName == "")
    return X2_EXECUTE_SCRIPT_MISSING;

  SetLocalInt(oItem, X2_L_LAST_ITEM_EVENT, nEvent);
  int nVal = ExecuteScriptAndReturnInt(sFileName, oItem);
  DeleteLocalInt(oItem, X2_L_LAST_ITEM_EVENT);
  return nVal;
}


// -----------------------------------------------------------------------------

// Tag based script execution for items.
// This is processed inline (ie: it uses ExecuteScript and not SignalEvent)
//
// Uses X2_ITEM_EVENT_* type events.
//
int ExecuteItemScript(object oItem, int nEvent);
int ExecuteItemScript(object oItem, int nEvent)
{
  string sScript = GetStringLowerCase(GetTag(oItem));
  return _ExecuteItemScript(sScript, oItem, nEvent);
}


// -----------------------------------------------------------------------------

// Execute a placeable user-defined script inline and not delayed.
//
// Use GetEvent to get event number
// Returns one of X2_EXECUTE_SCRIPT_* with X2_EXECUTE_SCRIPT_FAILURE
// being a soft failure
//
int ExecuteUserDefinedScript(int nEventId, object oTarget = OBJECT_SELF);
int ExecuteUserDefinedScript(int nEventId, object oTarget = OBJECT_SELF)
{
  if (!GetIsObjectValid(oTarget))
    return X2_EXECUTE_SCRIPT_FAILURE;

  int nScript = 0;
  switch (NWNX_Object_GetInternalObjectType(oTarget))
  {
    case NWNX_OBJECT_TYPE_INTERNAL_INVALID:
    case NWNX_OBJECT_TYPE_INTERNAL_STORE:
    case NWNX_OBJECT_TYPE_INTERNAL_WAYPOINT:
      return X2_EXECUTE_SCRIPT_FAILURE;

    case NWNX_OBJECT_TYPE_INTERNAL_ITEM:
      return ExecuteItemScript(oTarget, nEventId);

    case NWNX_OBJECT_TYPE_INTERNAL_AREA:         nScript = EVENT_SCRIPT_AREA_ON_USER_DEFINED_EVENT;         break;
    case NWNX_OBJECT_TYPE_INTERNAL_AREAOFEFFECT: nScript = EVENT_SCRIPT_AREAOFEFFECT_ON_USER_DEFINED_EVENT; break;
    case NWNX_OBJECT_TYPE_INTERNAL_CREATURE:     nScript = EVENT_SCRIPT_CREATURE_ON_USER_DEFINED_EVENT;     break;
    case NWNX_OBJECT_TYPE_INTERNAL_DOOR:         nScript = EVENT_SCRIPT_DOOR_ON_USERDEFINED;                break;
    case NWNX_OBJECT_TYPE_INTERNAL_ENCOUNTER:    nScript = EVENT_SCRIPT_ENCOUNTER_ON_USER_DEFINED_EVENT;    break;
    case NWNX_OBJECT_TYPE_INTERNAL_MODULE:       nScript = EVENT_SCRIPT_MODULE_ON_USER_DEFINED_EVENT;       break;
    case NWNX_OBJECT_TYPE_INTERNAL_PLACEABLE:    nScript = EVENT_SCRIPT_PLACEABLE_ON_USER_DEFINED_EVENT;    break;
    case NWNX_OBJECT_TYPE_INTERNAL_TRIGGER:      nScript = EVENT_SCRIPT_TRIGGER_ON_USER_DEFINED_EVENT;      break;

    default:
      return X2_EXECUTE_SCRIPT_FAILURE;
  }

  string sUserDefinedScript = GetEventScript(oTarget, nScript);
  if (sUserDefinedScript == "")
    return X2_EXECUTE_SCRIPT_CONTINUE;

  return ExecuteEventScript(sUserDefinedScript, nEventId, oTarget);
}

// void main() {}
