// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Area garbage collector.

Variables
~~~~~~~~~

:ev:`GC_AREA_MAX_AGE`: *(int)* Module and area variable, timeout in minutes before destroying leftover objects. Default: 10
*/


#include "fi_area"
#include "fi_events"
#include "fi_chat"
#include "fi_time"
#include "fi_utils"

#include "inc_array"

// -----------------------------------------------------------------------------

const string ARRAY_TRACKED_ITEMS = "GC_TRACKED_ITEMS";

const string DROPPED_TIMESTAMP = "GC_DROPPED_TIMESTAMP";

const string GC_AREA_MAX_AGE = "GC_AREA_MAX_AGE";

const int DEFAULT_MAX_AGE = 10;


// -----------------------------------------------------------------------------

void TrackItem(object oObject);
void TrackItem(object oObject)
{
  if (!GetIsObjectValid(oObject))
    return;

  if (!(
       (GetObjectType(oObject) == OBJECT_TYPE_ITEM) ||
       (GetObjectType(oObject) == OBJECT_TYPE_PLACEABLE && GetTag(oObject) == TAG_BODY_BAG) ||
       (GetObjectType(oObject) == OBJECT_TYPE_CREATURE  && GetTag(oObject) == TAG_BODY_BAG)
     ))
    return;

  if (Array_Contains_Obj(ARRAY_TRACKED_ITEMS, oObject))
    return;

  LogDebug("Area Garbage Collector: Tracking " + GetObjectInfo(oObject) + ".");

  SetLocalInt(oObject, DROPPED_TIMESTAMP, GetTimeStamp());
  Array_PushBack_Obj(ARRAY_TRACKED_ITEMS, oObject);
}


// -----------------------------------------------------------------------------
void TrackFromLocation(location lWhere);
void TrackFromLocation(location lWhere)
{
  // Mark the location for later.
  object oObject = GetFirstObjectInShape(SHAPE_SPHERE, 5.0f, lWhere, FALSE, OBJECT_TYPE_PLACEABLE);
  while (GetIsObjectValid(oObject))
  {
    TrackItem(oObject);
    oObject = GetNextObjectInShape(SHAPE_SPHERE, 5.0f, lWhere, FALSE, OBJECT_TYPE_PLACEABLE);
  }
}


// -----------------------------------------------------------------------------

void OnMinute(int nForceMaxAge = 0);
void OnMinute(int nForceMaxAge = 0)
{
  object oModule = GetModule();

  int nArraySize = Array_Size(ARRAY_TRACKED_ITEMS);

  int nIndex = 0;
  int nNow = GetTimeStamp();
  object oItem = Array_At_Obj(ARRAY_TRACKED_ITEMS, nIndex);
  while (oItem != OBJECT_INVALID)
  {
    if (InstructionLimitPassed("Area Garbage Collector"))
      return;

    if (!GetIsObjectValid(oItem))
    {
      Array_Erase(ARRAY_TRACKED_ITEMS, nIndex);
      oItem = Array_At_Obj(ARRAY_TRACKED_ITEMS, nIndex);
      continue;
    }

    int nStart = GetLocalInt(oItem, DROPPED_TIMESTAMP);
    if (nStart == 0)
    {
      LogDebug("Area Garbage Collector: Removal of " + GetObjectInfo(oItem) + " (no age)");
      Array_Erase(ARRAY_TRACKED_ITEMS, nIndex);
      oItem = Array_At_Obj(ARRAY_TRACKED_ITEMS, nIndex);
      continue;
    }

    int nMaxAge = GetLocalInt(GetArea(oItem), GC_AREA_MAX_AGE);
    if (nMaxAge <= 0)
      nMaxAge = GetLocalInt(oModule, GC_AREA_MAX_AGE);
    if (nMaxAge <= 0)
      nMaxAge = DEFAULT_MAX_AGE;
    if (nForceMaxAge > 0)
      nMaxAge = nForceMaxAge;
    if (nForceMaxAge < 0)
      nMaxAge = 0;
    nMaxAge = nMaxAge * 60;

    int nSeconds = nNow - nStart;
    if (nSeconds > 0 && nSeconds >= nMaxAge)
    {
      LogDebug("Area Garbage Collector: Removal of " + GetObjectInfo(oItem)
        + " (age: " + IntToString(nSeconds)
        + ", max age: " + IntToString(nMaxAge) + ")");
      DestroyObject(oItem, 0.0f);
      Array_Erase(ARRAY_TRACKED_ITEMS, nIndex);

      oItem = Array_At_Obj(ARRAY_TRACKED_ITEMS, nIndex);
      continue;
    }

    // LogDebug("Area Garbage Collector: Keeping " + GetObjectInfo(oItem)
    //   + " (age: " + IntToString(nSeconds)
    //   + ", max age: " + IntToString(nMaxAge) + ")");
    oItem = Array_At_Obj(ARRAY_TRACKED_ITEMS, ++nIndex);
  }

  // If there was items, print number of items now
  if (nArraySize)
    LogDebug("Area Garbage Collector: Heartbeat, " + IntToString(Array_Size(ARRAY_TRACKED_ITEMS)) + " items tracked.");
}


// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oPC = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  if (!(GetIsDebug(oPC) || GetIsDM(oPC) || GetIsPlayerDM(oPC)))
    return;

  SendMessageToPC(oPC, _RGB("/gc [MAX_AGE]: ", C1) + _RGB("Area Garbage collect", C2));

}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oPC = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  if (!(GetIsDebug(oPC) || GetIsDM(oPC) || GetIsPlayerDM(oPC)))
    return;

  string sCommand = GetStringTrim(sMessage);
  if (GetIsChatCommand(sCommand, "gc"))
  {
    sCommand = GetChatCommandPayload(sCommand, "gc");

    OnMinute(StringToInt(sCommand));

    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  object oArea = OBJECT_SELF;
  object oPC   = OBJECT_INVALID;
  object oItem = OBJECT_INVALID;

  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_ITEM_ACQUIRE);
      RegisterEvent(EVENT_ITEM_DROP_AFTER);

      RegisterEvent(EVENT_NPC_DEATH);

      RegisterEvent(EVENT_MODULE_FLOATING_MINUTE);

      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);

      RegisterEvent(EVENT_INFO_CONSOLE);
      break;

    case EVENT_ITEM_ACQUIRE:
      // We don't need to remove from the array, it'll be done automatically
      // on next loop.
      oItem = OBJECT_SELF;
      DeleteLocalInt(oItem, DROPPED_TIMESTAMP);
      break;

    case EVENT_ITEM_DROP_AFTER:
      oItem = StringToObject(NWNX_Events_GetEventData("ITEM"));
      TrackItem(oItem);
      break;

    case EVENT_NPC_DEATH:
      oPC = OBJECT_SELF; // technically, not a PC
      DelayCommand(10.0f, TrackFromLocation(GetLocation(oPC)));
      break;

    case EVENT_MODULE_FLOATING_MINUTE:
      OnMinute();
      break;

    case EVENT_INFO_CONSOLE: // TODO
      LogDebug("Area Garbage Collector: " + IntToString(Array_Size(ARRAY_TRACKED_ITEMS)) + " items tracked.");
      break;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
