// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*!
Area regeneration utility

This unit will destroy and recreate an area when the timer sets off. Timer is set
on last exit of an area by a PC.

Not only this action garbage collects items, but also resets encounters and containers.

Variables
~~~~~~~~~

:ev:`AREA_PREFIX`: *(string)* Prefix of all areas for the ``/recreate`` command.

:ev:`AREA_MAX_AGE`: *(int)* Module and area variable, timeout in minutes before destroying left over objects. Default: 30
*/


#include "fi_area"
#include "fi_chat"
#include "fi_events"
#include "fi_time"
#include "fi_utils"

#include "inc_array"


// -----------------------------------------------------------------------------

const string ARRAY_TRACKED_AREAS = "RECREATE_TRACKED_AREAS";

const string AREA_LAST_EXIT = "RECREATE_AREA_LAST_EXIT";

// in minutes
const string AREA_MAX_AGE = "AREA_MAX_AGE";

const int DEFAULT_MAX_AGE = 30;


// -----------------------------------------------------------------------------

void OnMinute(int nForceMaxAge = 0);
void OnMinute(int nForceMaxAge = 0)
{
  object oModule = GetModule();

  // Too dangerous when players are in transitions. Otherwise it can crash.
  if (GetHasPlayerInTransition())
  {
    DelayCommand(2.0f, OnMinute(nForceMaxAge));
    return;
  }

  int nArraySize = Array_Size(ARRAY_TRACKED_AREAS);

  int nNow = GetTimeStamp();

  int nSkipped = 0;
  int nIndex = 0;
  string sTag = Array_At_Str(ARRAY_TRACKED_AREAS, nIndex);
  while (sTag != "")
  {
    if (InstructionLimitPassed("Area Recreate"))
      return;

    object oArea = GetObjectByTag(sTag);
    object oStarting = GetAreaFromLocation(GetStartingLocation());
    if (oArea == oStarting)
    {
      // LogDebug("Area Recreate: Removing start area from tracked list.");
      Array_Erase(ARRAY_TRACKED_AREAS, nIndex);
      sTag = Array_At_Str(ARRAY_TRACKED_AREAS, nIndex);
    }

    if (NWNX_Area_GetNumberOfPlayersInArea(oArea) != 0)
    {
      // LogDebug("Area Recreate: " + GetObjectInfo(oArea) + ": players, skip");
      nSkipped += 1;
      sTag = Array_At_Str(ARRAY_TRACKED_AREAS, ++nIndex);
      continue;
    }

    int nStart = GetLocalInt(oArea, AREA_LAST_EXIT);
    if (nStart == 0)
    {
      // LogDebug("Area Recreate: " + GetObjectInfo(oArea) + ": nStart=0, skip");
      nSkipped += 1;
      sTag = Array_At_Str(ARRAY_TRACKED_AREAS, ++nIndex);
      continue;
    }

    int nMaxAge = GetLocalInt(oArea, AREA_MAX_AGE);
    if (nMaxAge <= 0)
      nMaxAge = GetLocalInt(oModule, AREA_MAX_AGE);
    if (nMaxAge <= 0)
      nMaxAge = DEFAULT_MAX_AGE;
    if (nForceMaxAge > 0)
      nMaxAge = nForceMaxAge;
    if (nForceMaxAge < 0)
      nMaxAge = 0;
    nMaxAge = nMaxAge * 60;

    int nSeconds = nNow - nStart;
    if (nSeconds > 0 && nSeconds >= nMaxAge)
    {
      LogNotice("Area Recreate: Recreating area " + GetObjectInfo(oArea) + ".");
      oArea = RecreateArea(GetResRef(oArea));
      if (GetIsObjectValid(oArea))
      {
        Array_Erase(ARRAY_TRACKED_AREAS, nIndex);
        sTag = Array_At_Str(ARRAY_TRACKED_AREAS, nIndex);
        continue;
      }
    }
    sTag = Array_At_Str(ARRAY_TRACKED_AREAS, ++nIndex);
  }

  if (nArraySize)
  {
    nArraySize =Array_Size(ARRAY_TRACKED_AREAS) - nSkipped;
    LogDebug("Area Recreate: Heartbeat, " + IntToString(nArraySize) + " areas tracked, "
      + IntToString(nSkipped) + " skipped");
  }
}


// -----------------------------------------------------------------------------

void TrackArea(object oArea);
void TrackArea(object oArea)
{
  if (!GetIsArea(oArea))
    return;

  SetLocalInt(oArea, AREA_LAST_EXIT, GetTimeStamp());

  string sTag = GetTag(oArea);
  if (Array_Contains_Str(ARRAY_TRACKED_AREAS, sTag))
    return;

  Array_PushBack_Str(ARRAY_TRACKED_AREAS, sTag);

  LogDebug("Area Recreate: Tracking " + GetObjectInfo(oArea) + " for a total of " + IntToString(Array_Size(ARRAY_TRACKED_AREAS)) + " areas tracked");
}


// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oPC = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  if (!(GetIsDebug(oPC) || GetIsDM(oPC) || GetIsPlayerDM(oPC)))
    return;

  SendMessageToPC(oPC, _RGB("/recreate [MAX_AGE|NAME]: ", C1) + _RGB("Regenerate all areas, or just an area", C2));
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oPC = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  if (!(GetIsDebug(oPC) || GetIsDM(oPC) || GetIsPlayerDM(oPC)))
    return;

  string sCommand = GetStringTrim(sMessage);
  if (GetIsChatCommand(sCommand, "recreate"))
  {
    string sResRef = GetChatCommandPayload(sCommand, "recreate");
    string sPrefix = GetLocalString(GetModule(), "AREA_PREFIX");

    // Is integer?
    if (GetIsStringInt(sResRef))
      OnMinute(StringToInt(sCommand));
    else
      RecreateArea(sResRef);

    RecreateArea(sPrefix + sResRef, oPC);
    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  object oArea = OBJECT_SELF;
  object oPC   = OBJECT_INVALID;
  object oItem = OBJECT_INVALID;

  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_AREA_EXIT);
      RegisterEvent(EVENT_MODULE_FLOATING_MINUTE);
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);

      RegisterEvent(EVENT_INFO_CONSOLE);
      break;

    case EVENT_MODULE_FLOATING_MINUTE:
      OnMinute();
      break;

    case EVENT_AREA_EXIT:
      oPC = GetEnteringObject();
      if (!GetIsPC(oPC))
        return;
      TrackArea(OBJECT_SELF);
      return;

    case EVENT_INFO_CONSOLE: // TODO

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
