// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst

Allow user to switch TLK to a different language.

Given TLK in the form of "<name>_en", "<name>_fr", etc. The following unit will
allow a user to switch to the TLK of his choice with the chat command "/lang".

If the given TLK does not exist, then an error will be printed.

On user client connection, this unit will try to set the correct TLK.
*/

// TODO: Show help only if there's something that can be changed


#include "fi_chat"
#include "fi_events"

#include "fi_pc"

#include "nwnx_player"
#include "nwnx_util"


const string RESNAME_TLK = "RESNAME_TLK";


// -----------------------------------------------------------------------------
void ChooseLanguage(object oPlayer, int nLanguage);
void ChooseLanguage(object oPlayer, int nLanguage)
{
  string sEnglishTlk = NWNX_Util_GetModuleTlkFile();
  string sBaseTlk = sEnglishTlk;

  if (nLanguage == PLAYER_LANGUAGE_ENGLISH)
  {
    NWNX_Player_SetResManOverride(oPlayer, RESTYPE_TLK, sEnglishTlk, "");
    NWNX_Player_ReloadTlk(oPlayer);
    // "Language set to ..."
    FloatingTextStrRefOnCreature(TLK_OFFSET + 10010, oPlayer, FALSE);
    return;
  }

  if (GetStringRight(sEnglishTlk, 3) == "_en")
    sBaseTlk = GetSubStringEx(sEnglishTlk, 0, GetStringLength(sEnglishTlk) - 3);

  string sLocaleTlk;
  string sLang;
  switch (nLanguage)
  {
    case PLAYER_LANGUAGE_FRENCH:  sLang = "FR"; sLocaleTlk = sBaseTlk + "_fr"; break;
    case PLAYER_LANGUAGE_GERMAN:  sLang = "DE"; sLocaleTlk = sBaseTlk + "_de"; break;
    case PLAYER_LANGUAGE_ITALIAN: sLang = "IT"; sLocaleTlk = sBaseTlk + "_it"; break;
    case PLAYER_LANGUAGE_SPANISH: sLang = "ES"; sLocaleTlk = sBaseTlk + "_es"; break;
    case PLAYER_LANGUAGE_POLISH:  sLang = "PL"; sLocaleTlk = sBaseTlk + "_pl"; break;
  }

  string sResRef = ResManFindPrefix(sLocaleTlk, RESTYPE_TLK, 1);
  if (sResRef == "")
  {
    SendMessageToPC(oPlayer, "Language '" + sLang + "' is not available.");
    return;
  }

  // TODO: Broadcast an event for other units
  NWNX_Player_SetResManOverride(oPlayer, RESTYPE_TLK, sEnglishTlk, sLocaleTlk);
  NWNX_Player_ReloadTlk(oPlayer);

  // "Language set to ..."
  FloatingTextStrRefOnCreature(TLK_OFFSET + 10010, oPlayer, FALSE);
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oPC = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  string sCommand = GetStringTrim(sMessage);
  if (GetIsChatCommand(sCommand, "lang"))
  {
    string sLang = GetChatCommandPayload(sCommand, "lang");

    sLang = GetStringLowerCase(sLang);
    if (sLang == "en")      ChooseLanguage(oPC, PLAYER_LANGUAGE_ENGLISH);
    else if (sLang == "fr") ChooseLanguage(oPC, PLAYER_LANGUAGE_FRENCH);
    else if (sLang == "de") ChooseLanguage(oPC, PLAYER_LANGUAGE_GERMAN);
    else if (sLang == "it") ChooseLanguage(oPC, PLAYER_LANGUAGE_ITALIAN);
    else if (sLang == "es") ChooseLanguage(oPC, PLAYER_LANGUAGE_SPANISH);
    else if (sLang == "pl") ChooseLanguage(oPC, PLAYER_LANGUAGE_POLISH);
    else
      SendMessageToPC(oPC, "error: unknown language");

    SetPCChatMessage("");
  }
}


// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  if (!GetIsDebug(oSpeaker))
    return;

  SendMessageToPC(oSpeaker, _RGB("/lang LANG", C1) + _RGB("Change TLK language (en, fr, de, it, es, pl).", C2));
}


// =============================================================================
void main()
{
  object oPC;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_CLIENT_ENTER);
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      RegisterEvent(EVENT_CLIENT_SEND_TLK_AFTER);
      return;

    case EVENT_CLIENT_SEND_TLK_AFTER: // TODO
      {
        string nLanguage = NWNX_Events_GetEventData("LANGUAGE");
      }
      break;

    case EVENT_CLIENT_ENTER:
      oPC = GetEnteringObject();
      ChooseLanguage(oPC, GetPlayerLanguage(oPC));
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
