// =============================================================================
// Framework Unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Store garbage collector.

Regularly check the time of non-infinite items in a store. If time is greater than :ev:`GC_STORE_MAX_AGE` variable, then
delete the item.

Variables
~~~~~~~~~

:ev:`GC_STORE_MAX_AGE`: *(int)* Store, area or module variable, checked in order. Timeout in realtime hours before destroying left over objects. Default: 28 days, 672 hours.
*/

#include "fi_events"

#include "fi_store"
#include "fi_time"
#include "fi_utils"


const string STORE_TIMESTAMP = "GC_STORE_TIMESTAMP";

const string GC_STORE_MAX_AGE = "GC_STORE_MAX_AGE";

const int DEFAULT_MAX_AGE = 672; // in hours


// -----------------------------------------------------------------------------
void OnDay();
void OnDay()
{
  object oModule = GetModule();
  int nNow = GetTimeStamp();

  int nIndex = 0;
  object oStore = Array_At_Obj(MOD_ARRAY_TRACKED_STORES, nIndex);
  while (GetIsObjectValid(oStore))
  {
    object oItem = GetFirstItemInInventory(oStore);
    while (GetIsObjectValid(oItem))
    {
      if (InstructionLimitPassed("Store Garbage Collector"))
        return;

      if (GetInfiniteFlag(oItem))
      {
        oItem = GetNextItemInInventory(oStore);
        continue;
      }

      int nStart = GetLocalInt(oItem, STORE_TIMESTAMP);
      if (nStart == 0)
      {
        // LogDebug("Store Garbage Collector: Addition of " + GetObjectInfo(oItem) + " from " + GetObjectInfo(oStore));
        SetLocalInt(oItem, STORE_TIMESTAMP, nNow);
        oItem = GetNextItemInInventory(oStore);
        continue;
      }

      int nMaxAge = GetLocalInt(oStore, GC_STORE_MAX_AGE);
      if (nMaxAge <= 0)
        nMaxAge = GetLocalInt(GetArea(oStore), GC_STORE_MAX_AGE);
      if (nMaxAge <= 0)
        nMaxAge = GetLocalInt(oModule, GC_STORE_MAX_AGE);
      if (nMaxAge <= 0)
        nMaxAge = DEFAULT_MAX_AGE;
      nMaxAge = nMaxAge * (60 * 60); // minutes * seconds

      int nSeconds = nNow - nStart;
      if (nSeconds > 0 && nSeconds >= nMaxAge)
      {
        LogDebug("Store Garbage Collector: Removal of " + GetObjectInfo(oItem)
          + " (age: " + IntToString(nSeconds)
          + ", max age: " + IntToString(nMaxAge) + ")"
          + " from " + GetObjectInfo(oStore));
        DestroyObject(oItem, 0.0f);

        oItem = GetNextItemInInventory(oStore);
        continue;
      }

      // LogDebug("Store Garbage Collector: Keeping of " + GetObjectInfo(oItem)
      //     + " (age: " + IntToString(nSeconds)
      //     + ", max age: " + IntToString(nMaxAge) + ")"
      //     + " from " + GetObjectInfo(oStore));
      oItem = GetNextItemInInventory(oStore);
    }

    oStore = Array_At_Obj(MOD_ARRAY_TRACKED_STORES, ++nIndex);
  }

}


// =============================================================================
void main()
{
  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_MODULE_REALTIME_HOUR);
      RegisterEvent(EVENT_ITEM_ACQUIRE);
      RegisterEvent(EVENT_ITEM_UNACQUIRE);
      return;

    case EVENT_MODULE_REALTIME_HOUR:
      OnDay();
      return;

    case EVENT_ITEM_ACQUIRE:   DeleteLocalInt(OBJECT_SELF, STORE_TIMESTAMP); break;
    case EVENT_ITEM_UNACQUIRE: DeleteLocalInt(OBJECT_SELF, STORE_TIMESTAMP); break;

    case EVENT_INFO_CONSOLE: // TODO
  }
}

