// =============================================================================
// Framework fire and forget
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Persistent stores.

Store will be saved in database with it's sResRef. Database name will be campaign name plus the suffix of the resref of the store. If the resref is not available, store will be saved in a generic database with tag as the tablename.

On area enter, store from database is merged into store from toolset, thus
taking the new settings, and infinite objects, and adding the non-infinite
objects.

Store is not meant to have non-infinite objects in the toolset store.
*/

#include "fi_console"
#include "fi_inventory"
#include "fi_events"

#include "fi_campaign"


const string ARRAY_OBJECTS = "OBJECTS";


const string TABLE_NAME = "store";


// Get the campaign name saved in module's variable "X0_CAMPAIGN_DB"
string GetCampaignNameEx(object oStore);
string GetCampaignNameEx(object oStore)
{
  string sResRef = GetResRef(oStore);
  return GetCampaignName(sResRef);
}


// -----------------------------------------------------------------------------
void MergeLeftIntoRight(object oSrc, object oDest);
void MergeLeftIntoRight(object oSrc, object oDest)
{
  // We don't need to copy properties since we take oSrc, the saved version,
  // and copy it's non-infinite objects into oDest

  // Remove all infinite items from oDest
  object oItem = GetFirstItemInInventory(oSrc);
  while (GetIsObjectValid(oItem))
  {
    if (!GetInfiniteFlag(oItem))
      CopyItem(oItem, oDest, TRUE);

    oItem = GetNextItemInInventory(oSrc);
  }

  DestroyInventory(oSrc, 0.1f);
  DestroyObject(oSrc, 0.1f);
}


// -----------------------------------------------------------------------------
void LoadObject(object oToolsetStore);
void LoadObject(object oToolsetStore)
{
  string sCampaign   = GetCampaignNameEx(oToolsetStore);
  string sTableName = TABLE_NAME;

  if (GetResRef(oToolsetStore) == "")
  {
    sCampaign = GetCampaignName("stores");
    sTableName = GetTag(oToolsetStore);
  }

  json jNewObject = GetCampaignJson(sCampaign, sTableName);
  if (JsonGetLength(jNewObject) == 0)
  {
    LogWarning("Persistent database '" + sCampaign + "', table '" + sTableName + "' not found.");
    return;
  }

  object oNewStore = JsonToObject(jNewObject, GetLocation(oToolsetStore));
  if (oNewStore == OBJECT_INVALID)
  {
    LogWarning("Persistent database '" + sCampaign + "', table '" + sTableName + "' is corrupt.");
    DeleteCampaignVariable(sCampaign, sTableName);
    return;
  }

  MergeLeftIntoRight(oNewStore, oToolsetStore);

  DestroyInventory(oNewStore, 0.1f);
  DestroyObject(oNewStore, 0.2f);

  LogNotice("Loaded persistent " + GetObjectInfo(oToolsetStore) + " and copied previous inventory over.");

}


// -----------------------------------------------------------------------------
// Search the area for persistent objects
void LoadArea(object oArea);
void LoadArea(object oArea)
{
  // TODO: Use JSON instead of array
  Array_Clear(ARRAY_OBJECTS);

  // Build an array first to avoid taking into account new stores.
  object oStore = GetFirstObjectInArea(oArea, OBJECT_TYPE_STORE);
  while (GetIsObjectValid(oStore))
  {
    Array_PushBack_Obj(ARRAY_OBJECTS, oStore);
    oStore = GetNextObjectInArea(oArea, OBJECT_TYPE_STORE);
  }

  while (Array_Size(ARRAY_OBJECTS))
  {
    oStore = Array_At_Obj(ARRAY_OBJECTS, 0);
    Array_Erase(ARRAY_OBJECTS, 0);
    LoadObject(oStore);
  }
}


// -----------------------------------------------------------------------------
void SaveObject(object oStore)
{
  string sCampaign = GetCampaignNameEx(oStore);
  string sTableName = TABLE_NAME;
  json jObject     = ObjectToJson(oStore);

  if (GetResRef(oStore) == "")
  {
    sCampaign = GetCampaignName("stores");
    sTableName = GetTag(oStore);
  }

  SetCampaignJson(sCampaign, sTableName, jObject);

  LogNotice("Saved persistent object " + GetObjectInfo(oStore) + ".");
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      GetRowCount(ARRAY_OBJECTS); // create the table
      RegisterEvent(EVENT_AREA_ENTER_ONCE);
      RegisterEvent(EVENT_STORE_CLOSE);
      return;

    case EVENT_AREA_ENTER_ONCE:
      LoadArea(OBJECT_SELF);
      return;

    case EVENT_STORE_CLOSE:
      // Event is after _SELL_AFTER, thus not taking in duplicates that
      // are deleted by the later.
      SaveObject(OBJECT_SELF);
      return;
  }
}
