// =============================================================================
// Framework event script
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
// Run through all the areas, and objects, and verify the settings, and set them
// if we can.


#include "fi_const"
#include "fi_campaign"
#include "fi_events"
#include "fi_expect"
#include "fi_time"


// -----------------------------------------------------------------------------

// Try to shutdown after module load
void _DelayedShutdown();
void _DelayedShutdown()
{
  NWNX_Administration_ShutdownServer();
  LogFatal("Server shutdown failed.");
}


// -----------------------------------------------------------------------------

// Verify required NWNX plugins, or LogFatal otherwise
int _PluginExists(string sPlugin);
int _PluginExists(string sPlugin)
{
  if (NWNX_PluginExists(sPlugin))
    return TRUE;
  LogFatal("NWNX plugin " + sPlugin + " must be loaded.");
  DelayCommand(0.0f, _DelayedShutdown());
  return FALSE;
}


// -----------------------------------------------------------------------------

// Verify required NWNX plugins, or LogFatal otherwise
void _NWNXKeyIsSet(string sKey);
void _NWNXKeyIsSet(string sKey)
{
  // https://nwnxee.github.io/unified/group__tweaks.html
  int nEnv = GetEnvironmentVariableAsBool(sKey);
  if (nEnv)
    return;

  LogFatal("Environment variable '" + sKey + "' must be set.");
  DelayCommand(0.0f, _DelayedShutdown());
}


// -----------------------------------------------------------------------------

void _RegisterEvents(string sRegex);
void _RegisterEvents(string sRegex)
{
  object oModule = GetModule();

  string sScript = NWNX_Util_GetFirstResRef(NWNX_UTIL_RESREF_TYPE_NCS, sRegex, TRUE);
  while (sScript != "")
  {
    LogNotice("Discovered script '" + sScript + "'.");
    ExecuteEventScript(sScript, EVENT_REGISTER, oModule);
    sScript = NWNX_Util_GetNextResRef();
  }
}


// -----------------------------------------------------------------------------
int ShutdownRequested();
int ShutdownRequested()
{
  object oModule = GetModule();

  // If env var 'NWNX_SHUTDOWN' is set, then shutdown.
  if (GetEnvironmentVariableAsBool(NWN_SHUTDOWN) || GetLocalInt(oModule, NWN_SHUTDOWN))
  {
    LogNotice("Shutting down because of NWN_SHUTDOWN was set.");
    NWNX_Administration_ShutdownServer();
    DelayCommand(0.0f, _DelayedShutdown());
    return TRUE;
  }
  return FALSE;
}


// =============================================================================
void main()
{
  object oModule = GetModule();
  SetLocalInt(oModule, SERVER_START_TIMESTAMP, GetTimeStamp());
  SetLocalInt(oModule, MODULE_START_TIMESTAMP, FALSE);

  ConsoleClearScreen();
  LogNotice("================================================================================");
  LogNotice("  Neverwinter Nights: Enhanced Edition");
  LogNotice("  Faerun event framework");
  LogNotice("================================================================================");
  LogNotice("Running initial commands...");

  // Verify that NWNX is loaded
  if (NWNX_Util_StripColors("OK") != "OK")
  {
    LogError("NWNX not activated.");
    NWNX_Administration_ShutdownServer();
    // Failure of the above is possible. Just return.
    LogFatal("Server shutdown failed.");
    DelayCommand(0.0f, _DelayedShutdown());
    return;
  }

  // Verify required NWNX plugins
  if (!_PluginExists("NWNX_Administration")) return;
  if (!_PluginExists("NWNX_Area")) return;
  if (!_PluginExists("NWNX_Creature")) return;
  if (!_PluginExists("NWNX_Encounter")) return;
  if (!_PluginExists("NWNX_Events")) return;
  if (!_PluginExists("NWNX_Feedback")) return;
  if (!_PluginExists("NWNX_Item")) return;
  if (!_PluginExists("NWNX_ItemProperty")) return;
  if (!_PluginExists("NWNX_Object")) return;
  if (!_PluginExists("NWNX_Player")) return;
  if (!_PluginExists("NWNX_SkillRanks")) return;
  if (!_PluginExists("NWNX_Util")) return;

  // Remove the cap on the instruction limit
  // This is an signed int32_t, max is 2147483647
  int nInstructionLimitBefore = NWNX_Util_GetInstructionLimit();
  NWNX_Util_SetInstructionLimit(2147483647);
  LogNotice("Module instruction limit has been set to " + IntToString(2147483647) + " for loading sequence." );

  // Move NWN_DEBUG from environment to module
  int nDebug = GetEnvironmentVariableAsBool(NWN_DEBUG);
  SetLocalInt(oModule, DEBUG, nDebug);
  LogNotice("Module DEBUG is " + (nDebug ? "TRUE" : "FALSE"), (nDebug ? CONSOLE_COLOR_RED : CONSOLE_COLOR_GREEN) );

  // Set ISATTY
  int nTTY = NWNX_Util_GetTTY() == "" ? FALSE : TRUE;
  LogNotice("Module TTY is " + (nTTY ? "TRUE" : "FALSE"), (nTTY ? CONSOLE_COLOR_RED : CONSOLE_COLOR_GREEN) );

  if (!nDebug && NWNX_PluginExists("NWNX_File"))
  {
    LogFatal("NWNX plugin NWNX_File must not be loaded in production.");
    DelayCommand(0.0f, _DelayedShutdown());
    return;
  }

  // Verify some important NWNX settings
  // https://nwnxee.github.io/unified/group__tweaks.html
  _NWNXKeyIsSet("NWNX_TWEAKS_DEAD_CREATURES_TRIGGER_ON_AREA_EXIT");
  _NWNXKeyIsSet("NWNX_TWEAKS_SEND_TLK_OVERRIDE_BEFORE_CHARGEN");

  // Set all scripts except those that we don't manage.
  ExpectScriptToExist(SCRIPT_FW_EVT_SPELLHOOK);
  SetLocalString(oModule, "X2_S_UD_SPELLSCRIPT", SCRIPT_FW_EVT_SPELLHOOK);

  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_MODULE_START,           "");

  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_ACQUIRE_ITEM,           SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_ACTIVATE_ITEM,          SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_CLIENT_ENTER,           SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_CLIENT_EXIT,            SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_EQUIP_ITEM,             SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_LOSE_ITEM,              SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_NUI_EVENT,              SCRIPT_FW_EVENT);
  // ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_CANCEL_CUTSCENE, SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_CHAT,            SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_DEATH,           SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_DYING,           SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_GUIEVENT,        SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_MODULE_LOAD,            SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_LEVEL_UP,        SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_REST,            SCRIPT_FW_EVENT);
  // ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_TARGET,      SCRIPT_FW_EVENT);
  // ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_TILE_ACTION, SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_RESPAWN_BUTTON_PRESSED, SCRIPT_FW_EVENT);
  ExpectAndSetScript(oModule, EVENT_SCRIPT_MODULE_ON_UNEQUIP_ITEM,           SCRIPT_FW_EVENT);

  // Print some module info
  LogNotice("Server name is set to '" + NWNX_Administration_GetServerName() + "'", CONSOLE_COLOR_CYAN);

  // Set the persistent campaign name
  string sCampaignName = GetStringLowerCase(GetTag(oModule));
  SetLocalString(oModule, X0_CAMPAIGN_DB, sCampaignName);
  LogNotice("Campaign name set to '" + sCampaignName + "'", CONSOLE_COLOR_CYAN);

  LogNotice("");
  LogNotice("================================================================================");
  LogNotice("                                 *** Loading ***                                ");
  LogNotice("================================================================================");
  LogNotice("");

  // Register all units, this sends a REGISTER event to the scripts
  // discovered.
  LogNotice("Registering framework AI scripts (fw_ai_*)...", CONSOLE_COLOR_CYAN);
  _RegisterEvents("fw_ai_.\*");
  LogNotice("Registered framework AI scripts (fw_ai_*).", CONSOLE_COLOR_CYAN);
  if (ShutdownRequested())

  LogNotice("Registering framework fire once scripts (fw_x_*)...", CONSOLE_COLOR_CYAN);
  _RegisterEvents("fw_x_.\*");
  LogNotice("Registered framework fire once scripts (fw_x_*).", CONSOLE_COLOR_CYAN);
  if (ShutdownRequested())
    return;

  LogNotice("Registering framework unit scripts (fu_*)...", CONSOLE_COLOR_CYAN);
  _RegisterEvents("fu_.\*");
  LogNotice("Registered framework unit scripts (fu_*).", CONSOLE_COLOR_CYAN);
  if (ShutdownRequested())
    return;

  LogNotice("Registering framework user-defined scripts (fd_*)...", CONSOLE_COLOR_CYAN);
  _RegisterEvents("fd_.\*");
  LogNotice("Registered framework user-defined scripts (fd_*).", CONSOLE_COLOR_CYAN);
  if (ShutdownRequested())
    return;

  LogNotice("Registering user unit scripts (un_*)...", CONSOLE_COLOR_CYAN);
  _RegisterEvents("un_.\*");
  LogNotice("Registered user unit scripts (un_*).", CONSOLE_COLOR_CYAN);
  if (ShutdownRequested())
    return;

  LogNotice("Registering user user-defined scripts (ud_*)...", CONSOLE_COLOR_CYAN);
  _RegisterEvents("ud_.\*");
  LogNotice("Registered user  user-defined scripts (ud_*).", CONSOLE_COLOR_CYAN);
  if (ShutdownRequested())
    return;

  // Print a fun fact
  int nInstructions = NWNX_Util_GetInstructionsExecuted();
  LogNotice("Executed " + IntToString(NWNX_Util_GetInstructionsExecuted()) + " instructions.");
  NWNX_Util_SetInstructionsExecuted(1);
  NWNX_Util_SetInstructionLimit(nInstructionLimitBefore); // 524288.
  LogNotice("Restored module instruction limit to " + IntToString(nInstructionLimitBefore) + "." );

  LogNotice("");
  LogNotice("================================================================================");
  LogNotice("                                 *** Loaded ***                                 ");
  LogNotice("================================================================================");

  // Save the module start time
  SetLocalInt(oModule, MODULE_START_TIMESTAMP, GetTimeStamp());
}

