// =============================================================================
// Framework fire and forget
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
// Dynamic placeables:
// - have a user defined script.
// - when tracked, the user defined script will be called every hour.
// - track and untrack will be called when those events happen
//
// TODO: If we wished to improve this script, we should store object + area in
// database, and clear the whole area when Untracking.

#include "fi_area"
#include "fi_events"
#include "fi_userdef"
#include "fi_utils"


const string ARRAY_TRACKED_OBJECTS = "MOD_TRACKED_OBJECTS";


// -----------------------------------------------------------------------------
void TrackPlaceables(object oArea);
void TrackPlaceables(object oArea)
{
  if (!GetIsArea(oArea))
    return;

  // If another player is in the area, then everything is already tracked.
  if (NWNX_Area_GetNumberOfPlayersInArea(oArea) > 1)
    return;

  int nOldSize = Array_Size(ARRAY_TRACKED_OBJECTS);
  object oObject = GetFirstObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
  while (GetIsObjectValid(oObject))
  {
    // It has to have a user-defined script
    string sScript = GetEventScript(oObject, EVENT_SCRIPT_PLACEABLE_ON_USER_DEFINED_EVENT);
    if (sScript == "")
    {
      oObject = GetNextObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
      continue;
    }

    // If already tracked, ignore
    if (Array_Contains_Obj(ARRAY_TRACKED_OBJECTS, oObject))
    {
      oObject = GetNextObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
      continue;
    }

    // LogDebug("Dynamic Placeables: Tracking " + GetObjectInfo(oObject));

    // Send a track event to the object
    ExecuteUserDefinedScript(EVENT_OBJECT_TRACKED, oObject);

    // Save it in database
    Array_PushBack_Obj(ARRAY_TRACKED_OBJECTS, oObject);

    oObject = GetNextObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
  }

  int nNewSize = Array_Size(ARRAY_TRACKED_OBJECTS);
  int nNew = nNewSize - nOldSize;
  LogDebug("Dynamic Placeables: Tracking " + IntToString(nNew)
    + " new objects for area " + GetObjectInfo(oArea)
    + " and a total of " + IntToString(nNewSize) + " objects.");
}


// -----------------------------------------------------------------------------
void UntrackPlaceables(object oArea);
void UntrackPlaceables(object oArea)
{
  if (!GetIsObjectValid(oArea))
    return;

  // All players must exit the area to stop tracking.
  if (NWNX_Area_GetNumberOfPlayersInArea(oArea) > 0)
    return;

  int nOldSize = Array_Size(ARRAY_TRACKED_OBJECTS);
  object oObject = GetFirstObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
  while (GetIsObjectValid(oObject))
  {
    // It has to have a user-defined script
    string sScript = GetEventScript(oObject, EVENT_SCRIPT_PLACEABLE_ON_USER_DEFINED_EVENT);
    if (sScript == "")
    {
      oObject = GetNextObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
      continue;
    }

    int nIndex = Array_Find_Obj(ARRAY_TRACKED_OBJECTS, oObject);
    if (nIndex >= 0)
    {
      Array_Erase(ARRAY_TRACKED_OBJECTS, nIndex);
      ExecuteUserDefinedScript(EVENT_OBJECT_UNTRACKED, oObject);
    }

    oObject = GetNextObjectInArea(oArea, OBJECT_TYPE_PLACEABLE);
  }

  int nNewSize = Array_Size(ARRAY_TRACKED_OBJECTS);
  int nNew = nOldSize - nNewSize;

  if (nNew)
  {
    LogDebug("Dynamic Placeables: Stopped tracking " + IntToString(nNew)
      + " objects for area " + GetObjectInfo(oArea)
      + " and tracking a total of " + IntToString(nNewSize) + " objects.");
  }
}


// -----------------------------------------------------------------------------
void BroadcastUserDefinedScript(int nEvent);
void BroadcastUserDefinedScript(int nEvent)
{
  int nIndex = 0;
  string sObject = Array_At_Str(ARRAY_TRACKED_OBJECTS, nIndex);
  while (sObject != "")
  {
    // TODO: if (InstructionLimitPassed("Dynamic Placeables"))
    //   return;

    object oObject = StringToObject(sObject);
    if (!GetIsObjectValid(oObject))
    {
      Array_Erase(ARRAY_TRACKED_OBJECTS, nIndex);
      sObject = Array_At_Str(ARRAY_TRACKED_OBJECTS, nIndex);
      continue;
    }

    ExecuteUserDefinedScript(nEvent, oObject);
    sObject = Array_At_Str(ARRAY_TRACKED_OBJECTS, ++nIndex);
  }
}


// -----------------------------------------------------------------------------
void UntrackAllAreas();
void UntrackAllAreas()
{
  object oArea = GetFirstArea();
  while (GetIsObjectValid(oArea))
  {
    UntrackPlaceables(oArea);
    oArea = GetNextArea();
  }
}


// =============================================================================
void main()
{
  object oPC = OBJECT_INVALID;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_MODULE_LOAD);

      RegisterEvent(EVENT_AREA_ENTER);
      RegisterEvent(EVENT_AREA_EXIT);
      RegisterEvent(EVENT_CALENDAR_HOUR);
      return;

    case EVENT_MODULE_LOAD:
      UntrackAllAreas();
      return;

    case EVENT_AREA_ENTER:
      oPC = GetEnteringObject();
      if (!GetIsPC(oPC))
        return;

      TrackPlaceables(OBJECT_SELF);
      return;

    case EVENT_AREA_EXIT:
      oPC = GetExitingObject();
      if (!GetIsPC(oPC))
        return;

      UntrackPlaceables(OBJECT_SELF);
      return;

    case EVENT_CALENDAR_HOUR:
      BroadcastUserDefinedScript(EVENT_OBJECT_TRACKED_CALENDAR_HOUR);
      return;
  }
}

