// =============================================================================
// Framework fire and forget
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

//! Verifies, as in unit testing, the different resref, tags and scripts configured
//! via the toolset.
//! The framework expects some of these to pass in order to function: eg, that every
//! area has the same resref and tag (in the toolset)


#include "fi_const"
#include "fi_events"
#include "fi_expect"
#include "fi_object"

#include "nwnx_area"

// =============================================================================
void main()
{
  object oModule = GetModule();

  if (!GetEnvironmentVariableAsBool(NWN_DEBUG))
    return;

  ExpectUnique(oModule);

  LogNotice("Verifying every area and object...");

  object oArea = GetFirstArea();
  while (oArea != OBJECT_INVALID)
  {
    // Set event scripts
    ExpectAndSetScript(oArea, EVENT_SCRIPT_AREA_ON_ENTER, SCRIPT_FW_EVENT);
    ExpectAndSetScript(oArea, EVENT_SCRIPT_AREA_ON_EXIT,  SCRIPT_FW_EVENT);

    ExpectIdenticalTagAndResRef(oArea);
    ExpectUnique(oArea);

    object oObject = GetFirstObjectInArea(oArea);
    while (oObject != OBJECT_INVALID)
    {
      int nObjectType = GetObjectType(oObject);
      string sTag = GetTag(oObject);

      // =======================================================================
      // STORE
      // =======================================================================
      if (nObjectType == OBJECT_TYPE_STORE)
      {
        ExpectIdenticalTagAndResRef(oObject);
        ExpectUnique(oObject);
        ExpectAndSetScript(oObject, EVENT_SCRIPT_STORE_ON_OPEN, SCRIPT_FW_EVENT);
        ExpectAndSetScript(oObject, EVENT_SCRIPT_STORE_ON_CLOSE, SCRIPT_FW_EVENT);

        object oItem = GetFirstItemInInventory(oObject);
        while (oItem != OBJECT_INVALID)
        {
          if (!GetInfiniteFlag(oItem))
            LogWarning(GetObjectInfo(oItem) + " from store " + GetObjectInfo(oObject) + " does not have infinite flag");
          oItem = GetNextItemInInventory(oObject);
        }
      }

      // =======================================================================
      // TRIGGER
      // =======================================================================
      if (nObjectType == NWNX_EVENTS_OBJECT_TYPE_TRIGGER)
      {
        // If user defined script is empty, then there's no point in having scripts.
        string sScript = GetEventScript(oObject, EVENT_SCRIPT_TRIGGER_ON_USER_DEFINED_EVENT);
        if (sScript == "")
        {
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_CLICKED, "");
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_HEARTBEAT, "");
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_OBJECT_ENTER, "");
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_OBJECT_EXIT, "");
        }
        else if (sScript != "")
        {
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_CLICKED, SCRIPT_FW_EVENT);
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_HEARTBEAT, SCRIPT_FW_EVENT);
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_OBJECT_ENTER, SCRIPT_FW_EVENT);
          ExpectAndSetScript(oObject, EVENT_SCRIPT_TRIGGER_ON_OBJECT_EXIT, SCRIPT_FW_EVENT);
        }
      }
      oObject = GetNextObjectInArea(oArea);
    }
    oArea = GetNextArea();
  }
}
