// =============================================================================
// Poison Weapon spellscript
// =============================================================================
//
// x2_s2_poisonwp
//
// Spell allows to add temporary poison properties
// to a melee weapon or stack of arrows
//
// The exact details of the poison are loaded from
// a 2da defined in x2_inc_itemprop X2_IP_POSIONWEAPON_2DA
// taken from the row that matches the last three letters
// of GetTag(GetSpellCastItem())

// Example: if an item is given the poison weapon property
//          and its tag ending on 004, the 4th row of the
//          2da will be used (1d2IntDmg DC14 18 seconds)
//
//          Rows 0 to 99 are bioware reserved
//
// Optionally, Non Assassins have a chance of poisoning themselves
// when handling an item with this spell
//
// Optionally, PCs with the feat can poison a weapon. Poisoned weapon will be poisoned for
// 1 hour, which helps make a Thief useable.
//
// https://nwn.fandom.com/wiki/Des_crft_poison.2da_(contents)
// https://nwn.fandom.com/wiki/Poison.2da_(contents)
//
// Spider    = Strength,     Poison Type = 0
// Centipede = Dexterity,    Poison Type = 1
// Giant Bee = Constitution, Poison Type = 2
//
// Restrictions
//  ... only weapons and ammo can be poisoned
//  ... restricted to piercing / slashing  damage
//
// Fix:
//   Now uses USE_POISON_FEAT_DURATION_ROUNDS to determine duration.

// TODO:

#include "fi_console"
#include "fi_events"

#include "nwnx_creature"

#include "x2_inc_itemprop"
#include "X2_inc_switches"


void main()
{
  object oItem   = GetSpellCastItem();
  object oPC     = OBJECT_SELF;
  object oTarget = GetSpellTargetObject();
  string sTag    = GetTag(oItem);

  if (oTarget == OBJECT_INVALID || GetObjectType(oTarget) != OBJECT_TYPE_ITEM)
  {
    FloatingTextStrRefOnCreature(83359, oPC); // "Invalid target "
    return;
  }

  int nType = GetBaseItemType(oTarget);
  if (!IPGetIsMeleeWeapon(oTarget) &&
      !IPGetIsProjectile(oTarget)  &&
       nType != BASE_ITEM_SHURIKEN &&
       nType != BASE_ITEM_DART     &&
       nType != BASE_ITEM_THROWINGAXE)
  {
    FloatingTextStrRefOnCreature(83359, oPC); // "Invalid target"
    return;
  }

  if (IPGetIsBludgeoningWeapon(oTarget))
  {
    FloatingTextStrRefOnCreature(83367, oPC); // "Weapon does not do slashing or piercing damage "
    return;
  }

  if (IPGetItemHasItemOnHitPropertySubType(oTarget, IP_CONST_ONHIT_ITEMPOISON))
  {
    FloatingTextStrRefOnCreature(83407, oPC); // "Weapon already poisoned"
    return;
  }

  // Get the 2da row to lookup the poison from the last three letters of the tag
  int nRow = StringToInt(GetStringRight(sTag, 3));
  if (nRow == 0)
  {
     FloatingTextStrRefOnCreature(83360,oPC); // "Nothing happens
     LogError("Item with tag " + GetTag(oItem) + " has the PoisonWeapon spellscript attached but tag does not contain 3 letter recipe code at the end!");
     return;
  }

  int nSaveDC     = StringToInt(Get2DAString(X2_IP_POISONWEAPON_2DA, "SaveDC", nRow));
  int nDuration   = StringToInt(Get2DAString(X2_IP_POISONWEAPON_2DA, "Duration", nRow));
  int nPoisonType = StringToInt(Get2DAString(X2_IP_POISONWEAPON_2DA, "PoisonType", nRow));
  int nApplyDC    = StringToInt(Get2DAString(X2_IP_POISONWEAPON_2DA, "ApplyCheckDC", nRow));

  int nNewDuration = GetLocalInt(GetModule(), "USE_POISON_FEAT_DURATION_ROUNDS");
  if (nNewDuration)
    nDuration = FloatToInt(RoundsToSeconds(nNewDuration));

  // Always force attacks of opportunity
  NWNX_Creature_BroadcastAttackOfOpportunity(oPC);

  // Make all calculations here.
  int bHasFeat = GetHasFeat(FEAT_USE_POISON, oPC);

  // Poison restricted to feat
  if (GetModuleSwitchValue(MODULE_SWITCH_RESTRICT_USE_POISON_TO_FEAT) && !bHasFeat) // X2_SWITCH_RESTRICT_USE_POISON_FEAT
  {
    FloatingTextStrRefOnCreature(84420, oPC); // You cannot use this item.
    return;
  }

  int nDex = GetAbilityModifier(ABILITY_DEXTERITY, oPC);
  int nCheck = 10 + d10() + nDex;
  if (GetIsInCombat(oPC))
    nCheck = d20() + nDex;

  LogInfo(GetObjectInfo(oPC) + " is applying poison "
    + GetObjectInfo(oItem) + " type " + IntToString(nPoisonType) + " (row: " + IntToString(nRow) + ") to weapon; skill: " + IntToString(nCheck) + " vs DC: " + IntToString(nApplyDC));


  if (nCheck < nApplyDC)
  {
    FloatingTextStrRefOnCreature(83368, oPC); // "Failed"
    return;
  }

  itemproperty ip = ItemPropertyOnHitProps(IP_CONST_ONHIT_ITEMPOISON, nSaveDC, nPoisonType);
  IPSafeAddItemProperty(oTarget, ip, IntToFloat(nDuration), X2_IP_ADDPROP_POLICY_KEEP_EXISTING, TRUE, TRUE);

  // TODO:
  // technically this is not 100% safe but since there is no way to retrieve the sub
  // properties of an item (i.e. itempoison), there is nothing we can do about it
  if (IPGetItemHasItemOnHitPropertySubType(oTarget, IP_CONST_ONHIT_ITEMPOISON))
  {
    FloatingTextStrRefOnCreature(83361, oPC); // "Weapon is coated with poison"

    effect eVis = EffectVisualEffect(VFX_IMP_PULSE_NATURE);
    IPSafeAddItemProperty(oTarget, ItemPropertyVisualEffect(ITEM_VISUAL_ACID), IntToFloat(nDuration), X2_IP_ADDPROP_POLICY_KEEP_EXISTING, TRUE, FALSE);
    ApplyEffectToObject(DURATION_TYPE_INSTANT, eVis, GetItemPossessor(oTarget));
  }
  else
    FloatingTextStrRefOnCreature(83360, oPC); // "Nothing happens"
}
